import {
  require_disposable,
  require_main,
  require_messageReader,
  require_messageWriter,
  require_messages
} from "./chunk-WGZQEPBW.js";
import {
  __commonJS
} from "./chunk-3DB52H5I.js";

// ../node_modules/@codingame/monaco-jsonrpc/lib/disposable.js
var require_disposable2 = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/disposable.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DisposableCollection = exports.Disposable = void 0;
    var disposable_1 = require_disposable();
    Object.defineProperty(exports, "Disposable", { enumerable: true, get: function() {
      return disposable_1.Disposable;
    } });
    var DisposableCollection = class {
      constructor() {
        this.disposables = [];
      }
      dispose() {
        while (this.disposables.length !== 0) {
          this.disposables.pop().dispose();
        }
      }
      push(disposable) {
        const disposables = this.disposables;
        disposables.push(disposable);
        return {
          dispose() {
            const index = disposables.indexOf(disposable);
            if (index !== -1) {
              disposables.splice(index, 1);
            }
          }
        };
      }
    };
    exports.DisposableCollection = DisposableCollection;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/socket/socket.js
var require_socket = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/socket/socket.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/socket/reader.js
var require_reader = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/socket/reader.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.WebSocketMessageReader = void 0;
    var messageReader_1 = require_messageReader();
    var WebSocketMessageReader = class extends messageReader_1.AbstractMessageReader {
      constructor(socket) {
        super();
        this.socket = socket;
        this.state = "initial";
        this.events = [];
        this.socket.onMessage((message) => this.readMessage(message));
        this.socket.onError((error) => this.fireError(error));
        this.socket.onClose((code, reason) => {
          if (code !== 1e3) {
            const error = {
              name: "" + code,
              message: `Error during socket reconnect: code = ${code}, reason = ${reason}`
            };
            this.fireError(error);
          }
          this.fireClose();
        });
      }
      listen(callback) {
        if (this.state === "initial") {
          this.state = "listening";
          this.callback = callback;
          while (this.events.length !== 0) {
            const event = this.events.pop();
            if (event.message) {
              this.readMessage(event.message);
            } else if (event.error) {
              this.fireError(event.error);
            } else {
              this.fireClose();
            }
          }
        }
        return {
          dispose: () => {
            if (this.callback === callback) {
              this.callback = void 0;
            }
          }
        };
      }
      readMessage(message) {
        if (this.state === "initial") {
          this.events.splice(0, 0, { message });
        } else if (this.state === "listening") {
          const data = JSON.parse(message);
          this.callback(data);
        }
      }
      fireError(error) {
        if (this.state === "initial") {
          this.events.splice(0, 0, { error });
        } else if (this.state === "listening") {
          super.fireError(error);
        }
      }
      fireClose() {
        if (this.state === "initial") {
          this.events.splice(0, 0, {});
        } else if (this.state === "listening") {
          super.fireClose();
        }
        this.state = "closed";
      }
    };
    exports.WebSocketMessageReader = WebSocketMessageReader;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/socket/writer.js
var require_writer = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/socket/writer.js"(exports) {
    "use strict";
    var __awaiter = exports && exports.__awaiter || function(thisArg, _arguments, P, generator) {
      function adopt(value) {
        return value instanceof P ? value : new P(function(resolve) {
          resolve(value);
        });
      }
      return new (P || (P = Promise))(function(resolve, reject) {
        function fulfilled(value) {
          try {
            step(generator.next(value));
          } catch (e) {
            reject(e);
          }
        }
        function rejected(value) {
          try {
            step(generator["throw"](value));
          } catch (e) {
            reject(e);
          }
        }
        function step(result) {
          result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected);
        }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
      });
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.WebSocketMessageWriter = void 0;
    var messageWriter_1 = require_messageWriter();
    var WebSocketMessageWriter = class extends messageWriter_1.AbstractMessageWriter {
      constructor(socket) {
        super();
        this.socket = socket;
        this.errorCount = 0;
      }
      end() {
      }
      write(msg) {
        return __awaiter(this, void 0, void 0, function* () {
          try {
            const content = JSON.stringify(msg);
            this.socket.send(content);
          } catch (e) {
            this.errorCount++;
            this.fireError(e, msg, this.errorCount);
          }
        });
      }
    };
    exports.WebSocketMessageWriter = WebSocketMessageWriter;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/socket/connection.js
var require_connection = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/socket/connection.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.createWebSocketConnection = void 0;
    var vscode_jsonrpc_1 = require_main();
    var reader_1 = require_reader();
    var writer_1 = require_writer();
    function createWebSocketConnection(socket, logger) {
      const messageReader = new reader_1.WebSocketMessageReader(socket);
      const messageWriter = new writer_1.WebSocketMessageWriter(socket);
      const connection = vscode_jsonrpc_1.createMessageConnection(messageReader, messageWriter, logger);
      connection.onClose(() => connection.dispose());
      return connection;
    }
    exports.createWebSocketConnection = createWebSocketConnection;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/socket/index.js
var require_socket2 = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/socket/index.js"(exports) {
    "use strict";
    var __createBinding = exports && exports.__createBinding || (Object.create ? function(o, m, k, k2) {
      if (k2 === void 0)
        k2 = k;
      Object.defineProperty(o, k2, { enumerable: true, get: function() {
        return m[k];
      } });
    } : function(o, m, k, k2) {
      if (k2 === void 0)
        k2 = k;
      o[k2] = m[k];
    });
    var __exportStar = exports && exports.__exportStar || function(m, exports2) {
      for (var p in m)
        if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports2, p))
          __createBinding(exports2, m, p);
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    __exportStar(require_socket(), exports);
    __exportStar(require_reader(), exports);
    __exportStar(require_writer(), exports);
    __exportStar(require_connection(), exports);
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/logger.js
var require_logger = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/logger.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.ConsoleLogger = void 0;
    var ConsoleLogger = class {
      error(message) {
        console.error(message);
      }
      warn(message) {
        console.warn(message);
      }
      info(message) {
        console.info(message);
      }
      log(message) {
        console.log(message);
      }
      debug(message) {
        console.debug(message);
      }
    };
    exports.ConsoleLogger = ConsoleLogger;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/connection.js
var require_connection2 = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/connection.js"(exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.toSocket = exports.listen = void 0;
    var socket_1 = require_socket2();
    var logger_1 = require_logger();
    function listen(options) {
      const { webSocket, onConnection } = options;
      const logger = options.logger || new logger_1.ConsoleLogger();
      webSocket.onopen = () => {
        const socket = toSocket(webSocket);
        const connection = socket_1.createWebSocketConnection(socket, logger);
        onConnection(connection);
      };
    }
    exports.listen = listen;
    function toSocket(webSocket) {
      return {
        send: (content) => webSocket.send(content),
        onMessage: (cb) => webSocket.onmessage = (event) => cb(event.data),
        onError: (cb) => webSocket.onerror = (event) => {
          if ("message" in event) {
            cb(event.message);
          }
        },
        onClose: (cb) => webSocket.onclose = (event) => cb(event.code, event.reason),
        dispose: () => webSocket.close()
      };
    }
    exports.toSocket = toSocket;
  }
});

// ../node_modules/@codingame/monaco-jsonrpc/lib/index.js
var require_lib = __commonJS({
  "../node_modules/@codingame/monaco-jsonrpc/lib/index.js"(exports) {
    "use strict";
    var __createBinding = exports && exports.__createBinding || (Object.create ? function(o, m, k, k2) {
      if (k2 === void 0)
        k2 = k;
      Object.defineProperty(o, k2, { enumerable: true, get: function() {
        return m[k];
      } });
    } : function(o, m, k, k2) {
      if (k2 === void 0)
        k2 = k;
      o[k2] = m[k];
    });
    var __exportStar = exports && exports.__exportStar || function(m, exports2) {
      for (var p in m)
        if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports2, p))
          __createBinding(exports2, m, p);
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    __exportStar(require_main(), exports);
    __exportStar(require_messages(), exports);
    __exportStar(require_disposable2(), exports);
    __exportStar(require_socket2(), exports);
    __exportStar(require_logger(), exports);
    __exportStar(require_connection2(), exports);
  }
});

// dep:@codingame_monaco-jsonrpc
var codingame_monaco_jsonrpc_default = require_lib();
export {
  codingame_monaco_jsonrpc_default as default
};
//# sourceMappingURL=@codingame_monaco-jsonrpc.js.map
